

from typing import Final


class Const:
    DATETIME_FORMAT: Final[str] = "%Y%m%d%H%M%S"

    ALLOW_EDITING_DELTA: Final[int] = 47
    SECONDS_BETWEEN_OPERATION: Final[int] = 1

    STABLE_WAIT_TIME_SECONDS: Final[float] = 0.7
