

from .exceptions import (
    DifferentButtonsInMatrix,
    UnknownButtonType,
    NoOneButtonParamIsFilled,
    TooMoreCharacters,
    RecursionObjectParse,
    WrongObjectType,
    CantEditMedia,
    BotIsUndefined,
    MessageTooOld,
    MessageTextIsNone,
    UndefinedMagicFilterModel,
    UndefinedType,
    CallbackEventWasNotGiven,
    Throttling
)
